/*************************************************************************
@@DDSLoader.cpp

@@Direct Draw Surface Loader

@@Date : Nov. 01, 2007
@@Version : 1.0
@@Author : Pocol
@@Memo :
*************************************************************************/

//
//@include
//
#include <fstream>
#include "DDSLoader.h"
using namespace std;

//@disable Warning C4996
#pragma warning (disable : 4996 )


//////////////////////////////////////////////////////////////////////////
//@@DDSImage
//////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------------------------------
//@@DDSImage
//@@Desc : RXgN^
//-----------------------------------------------------------------------------------------------------
DDSImage::DDSImage()
{
	imageSize = 0;
	imageData = NULL;
	format = GL_RGB;
	internalFormat = GL_RGB;
	width = 0;
	height = 0;
	bpp = 0;
	ID = 0;
	numMipmaps = 0;
}

//-----------------------------------------------------------------------------------------------------
//@@~DDSImage
//@@Desc : fXgN^
//-----------------------------------------------------------------------------------------------------
DDSImage::~DDSImage()
{
	if ( imageData )
	{
		delete [] imageData;
		imageData = NULL;
	}
}

//-----------------------------------------------------------------------------------------------------
//@@ReadFileDDS
//@@Desc : DDSt@C̓ǂݍ
//-----------------------------------------------------------------------------------------------------
bool DDSImage::ReadDDS(const char *filename)
{
	FILE *fp;
	char magic[4];
	int mipmapFactor;
	long curr, end;
	DDSurfaceDesc ddsd;

	//@GLEW̏
	if ( glewInit() != GLEW_OK )
	{
		cout << "Error : GLEW̏Ɏs܂\n";
		return false;
	}

	//@t@CJ
	fp = fopen(filename, "rb");
	if ( !fp )
	{
		cout << "Error : w肳ꂽt@CJ܂ł\n";
		cout << "File : " << filename << endl;
		return false;
	}

	//@}WbNǂݎ
	fread(&magic, sizeof(char), 4, fp);

	//@}WbN`FbN
	if ( strncmp(magic, "DDS ", 4 ) != 0 )
	{
		cout << "Error : DDSt@Cł͂܂\n";
		fclose(fp);
		return false;
	}

	//@wb_[ǂݎ
	fread(&ddsd, sizeof(ddsd), 1, fp);

	//@Ei[
	height = ddsd.height;
	width = ddsd.width;
	numMipmaps = ddsd.mipMapLevels;

	//@tH[}bg
	switch ( ddsd.format.fourCC )
	{
	case FOURCC_DXT1:
		//@DXT1
		format = GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
		internalFormat = 3;
		mipmapFactor = 2;
		break;

	case FOURCC_DXT3:
		//@DXT3
		format = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
		internalFormat = 4;
		mipmapFactor = 4;
		break;

	case FOURCC_DXT5:
		//@DXT5
		format = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
		internalFormat = 4;
		mipmapFactor = 4;
		break;

	default:
		cout << "Error : Ή̌`łBDXT1, DXT3, DXT5̂ꂩgpĂ\n";
		return false;
	}

	//@eNZf[^̃TCYZo
	curr = ftell(fp);
	fseek(fp, 0, SEEK_END);
	end = ftell(fp);
	fseek(fp, curr, SEEK_SET);
	imageSize = end - curr;
	imageData = new GLubyte [imageSize];

	//@sNZf[^̓ǂݍ
	fread(imageData, sizeof(GLubyte), imageSize, fp);

	//@t@C
	fclose(fp);

	return true;
}

//-----------------------------------------------------------------------------------------------------
//@@DecompressDDS
//@@Desc : S3TCk̉𓀍
//-----------------------------------------------------------------------------------------------------
void DDSImage::DecompressDDS()
{
	int blockSize;
	int offset = 0;
	GLsizei mWidth = width, mHeight = height, mSize = 0;

	//@DXT1
	if ( format == GL_COMPRESSED_RGBA_S3TC_DXT1_EXT )
		blockSize = 8;
	//@DXT3, DXT5
	else
		blockSize = 16;

	//@
	for ( int i=0; i<(int)numMipmaps; i++ )
	{
		mSize = ( (mWidth+3)/4 ) * ( (mHeight+3)/4 ) * blockSize;
		glCompressedTexImage2D(GL_TEXTURE_2D, i, format, mWidth, mHeight, 0, mSize, imageData + offset );
		
		if ( mWidth >> 1 )	mWidth = (mWidth >> 1);
		else	mWidth = 1;

		if ( mHeight >> 1 )	mHeight = (mHeight >> 1);
		else	mHeight = 1;

		offset += mSize;
	}
}

//-----------------------------------------------------------------------------------------------------
//@@Load
//@@Desc : DDSt@Cǂݍ݁CeNX`𐶐
//-----------------------------------------------------------------------------------------------------
GLuint DDSImage::Load(const char *filename)
{
	//@t@C̓ǂݍ
	if ( !ReadDDS(filename) )
		return 0;

	//@eNX`𐶐
	glGenTextures(1, &ID);

	//@eNX`oCh
	glBindTexture(GL_TEXTURE_2D, ID);

	//@gEk@̎w
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	
	//@eNX`̐ݒ
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	//@𓀍
	DecompressDDS();

	if ( imageData )
	{
		delete [] imageData;
		imageData = NULL;
	}

	glBindTexture(GL_TEXTURE_2D, 0);

	//@eNX`ԍԂ
	return ID;
}