/************************************************************************
@@Fire.cpp

  @@Fire Effect

	@@Date : Nov. 12, 2007
	  @@Version : 1.0
		@@Author : Pocol
		  ************************************************************************/

//
//@include
//
#include "Fire.h"

//
//@global
//
extern double Random();
extern double Random05();
const static double InitXZ = 0.0;
const static double InitY = -1.0;
const static double MinSize = 0.12;
const static double MaxSize = 0.20;
const static double MinVY = 0.03;
const static double MaxVY = 0.06;
const static double RandColor = 0.1;
const static int MinTime = 10;
const static int MaxTime = 40;

/////////////////////////////////////////////////////////////////////////
//@@FireParticle
/////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------
//@@FireParticle
//@@Desc : RXgN^
//---------------------------------------------------------------------------------------------------
FireParticle::FireParticle(float base_color[], double max_xz) : maxXZ(max_xz)
{
	for ( int i=0; i<3; i++ )
		baseColor[i] = base_color[i];

	Initialize();

	for ( int i=0, n=Random()*MaxTime; i<n; i++ ) Move();
}

//----------------------------------------------------------------------------------------------------
//@@Initialize
//@@Desc : 
//----------------------------------------------------------------------------------------------------
void FireParticle::Initialize()
{
	double rad = Random()*3.14*2.0;
	double xz = Random05()*maxXZ;

	position[0] = InitXZ + cos(rad)*xz;
	position[1] = InitY;
	position[2] = InitXZ + sin(rad)*xz;

	velocity[0] = 0.0;
	velocity[1] = MinVY + (MaxVY - MinVY)*Random();
	velocity[2] = 0.0;

	size = MinSize + (MaxSize - MinSize)*Random();

	float c = Random05()*RandColor;
	color[0] = baseColor[0] + c;
	color[1] = baseColor[1] + c;
	color[2] = baseColor[2] + c;

	time = 0;
	endTime = MinTime + (MaxTime - MinTime)*Random();
}

//----------------------------------------------------------------------------------------------------
//@@Move
//@@Desc : ړ
//----------------------------------------------------------------------------------------------------
void FireParticle::Move()
{
	for ( int i=0; i<3; i++ )
	{
		position[i] += velocity[i];
	}
	time++;

	if ( time >= endTime ) Initialize();
}

//----------------------------------------------------------------------------------------------------
//@@Render
//@@Desc : `揈
//----------------------------------------------------------------------------------------------------
void FireParticle::Render(double argAlpha)
{
	GLfloat mat_color[4] = { color[0], color[1], color[2], sin(3.14*(float)(endTime - time)/endTime) - argAlpha};
	glColor4fv(mat_color);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_color);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_color);
	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, mat_color);

	glBegin(GL_QUADS);
	glTexCoord2f(0.0, 0.0);	glVertex3d(position[0] - size, position[1] - size, position[2]);
	glTexCoord2f(0.0, 1.0);	glVertex3d(position[0] - size, position[1] + size, position[2]);
	glTexCoord2f(1.0, 1.0);	glVertex3d(position[0] + size, position[1] + size, position[2]);
	glTexCoord2f(1.0, 0.0);	glVertex3d(position[0] + size, position[1] - size, position[2]);
	glEnd();
}

/////////////////////////////////////////////////////////////////////////
//@@Fire class
/////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------------------------------
//@@Fire
//@@Desc : RXgN^
//-----------------------------------------------------------------------------------------------------
Fire::Fire(int fire_count, float fire_color[], double max_xz) : fireCount(fire_count)
{
	fire_particle = new FireParticle *[fireCount];
	for ( int i=0; i<fireCount; i++ )
	{
		fire_particle[i] = new FireParticle(fire_color, max_xz);
	}
}

//-----------------------------------------------------------------------------------------------------
//@@~Fire
//@@Desc : fXgN^
//-----------------------------------------------------------------------------------------------------
Fire::~Fire()
{
	delete [] fire_particle;
}

//----------------------------------------------------------------------------------------------------
//@@LoadTexture
//@@Desc : eNX`̃[h
//----------------------------------------------------------------------------------------------------
void Fire::LoadTexture(const char *filename)
{
	texture.Load(filename);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
}

//-----------------------------------------------------------------------------------------------------
//@@Move
//@@Desc : ړ
//-----------------------------------------------------------------------------------------------------
void Fire::Move()
{
	for ( int i=0; i<fireCount; i++ )
	{
		fire_particle[i]->Move();
	}
}

//---------------------------------------------------------------------------------------------------
//@@Render
//@@Desc : `揈
//---------------------------------------------------------------------------------------------------
void Fire::Render(double argAlpha)
{
	glDisable(GL_DEPTH_TEST);

	/*
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glEnable(GL_BLEND);
	*/

	glBlendFunc(GL_ONE, GL_ONE);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture.ID);

	for ( int i=0; i<fireCount; i++ )
	{
		fire_particle[i]->Render(argAlpha);
	}

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glDisable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 0);

	//glDisable(GL_BLEND);

	glEnable(GL_DEPTH_TEST);
}

/////////////////////////////////////////////////////////////////////////
//@@FireModel class
/////////////////////////////////////////////////////////////////////////

FireModel::FireModel(int argFadeInTime, int argNonFadeTime, int argFadeOutTime)
{
	static GLfloat	fireCol[3] = { 0.06f, 0.01f, 0.00f };
	fire = new Fire(280, fireCol, 2.2);
	setShape(&dummyShape);	// `悪XLbv̂h~

	alpha = 0.0;
	timer = 0;
	fadeInTime = argFadeInTime;
	nonFadeTime = argNonFadeTime;
	fadeOutTime = argFadeOutTime;
}

FireModel::~FireModel()
{
	delete fire;
}

void FireModel::init()
{
	//@GLEW̏
	if ( glewInit() != GLEW_OK )
	{
		cout << "Error : GLEW̏Ɏs܂\n";
		exit(0);
	}

	fire->LoadTexture("Data/CG/sample.dds");
}

bool FireModel::isLimit()
{
	if(timer >= fadeInTime + nonFadeTime + fadeOutTime)	return true;

	return false;
}

void FireModel::preShader()
{
	if(timer < fadeInTime)	alpha -= 1.0 / fadeInTime;
	else if(timer < fadeInTime + nonFadeTime)	alpha = 0.0;
	else if(timer < fadeInTime + nonFadeTime + fadeOutTime)	alpha += 1.0 / fadeOutTime;

	fire->Move();
	fire->Render(alpha);
}

void FireModel::postShader()
{
	++timer;
}